/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.ftp.internal;

import java.io.IOException;
import java.net.*;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.ftp.FtpException;

public class PortDataConnection extends DataConnection {
	private InetAddress address;
	private int port;
	private ServerSocket serverSocket;

	/**
	 * Creates a data transfer manager.
	 * @param worker the worker for the thread
	 */
	public PortDataConnection() {
	}
	
	public InetAddress getLocalAddress() {
		return address;
	}
	
	public int getLocalPort() {
		return port;
	}
	
	public void open(IProgressMonitor monitor) throws FtpException {
		serverSocket = openServerSocket();
		super.open(monitor);
	}
	
	public void close(boolean interrupt) throws FtpException {
		try {
			super.close(interrupt);
		} finally {
			if (interrupt) {
				try {
					closeServerSocket();
				} catch (Exception e) {
					// eat any exceptions resulting from forced closure of the socket
				}
			} else {
				closeServerSocket();
			}
		}
	}
	
	protected ServerSocket openServerSocket() throws FtpException {
		try {
			ServerSocket serverSocket = new ServerSocket(0 /*any port*/, 1 /*backlog*/);
			serverSocket.setSoTimeout(FtpPlugin.getDefault().getPortConnectionTimeout() * 1000);
			address = InetAddress.getLocalHost(); // FIXME: won't work well for multi-homed hosts
			port = serverSocket.getLocalPort();
			return serverSocket;
		} catch (IOException e) {
			throw new FtpCommunicationException(FtpPlugin.getResourceString("FTPClient.ErrorOpeningPortServerSocket"), e); //$NON-NLS-1$
		}
	}
	
	protected synchronized void closeServerSocket() throws FtpException {
		if (serverSocket == null) return;
		try {
			serverSocket.close();
		} catch (IOException e) {
			throw new FtpCommunicationException(FtpPlugin.getResourceString("FTPClient.ErrorClosingPortServerSocket"), e); //$NON-NLS-1$
		} finally {
			serverSocket = null;
		}
	}
	
	protected Socket openSocket() throws FtpException {
		try {
			ServerSocket ss;
			synchronized (this) {
				ss = serverSocket;
				if (ss == null) {
					// this can happen if we interrupt the thread
					throw new FtpCommunicationException(FtpPlugin.getResourceString("FTPClient.CannotOpenPortDataConnection"), null); //$NON-NLS-1$
				}
			}
			return ss.accept();
		} catch (IOException e) {
			throw new FtpCommunicationException(FtpPlugin.getResourceString("FTPClient.CannotOpenPortDataConnection"), e); //$NON-NLS-1$
		} finally {
			closeServerSocket();
		}
	}	
}
