/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.target;

import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.core.runtime.PlatformObject;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.target.internal.Utils;

/**
 * An abstract class that provides some helpful parts of an
 * {@link TargetResource} implementation/
 * <p>
 * <b>Note:</b> This class/interface is part of an interim API that is still under 
 * development and expected to change significantly before reaching stability. 
 * It is being made available at this early stage to solicit feedback from pioneering 
 * adopters on the understanding that any code that uses this API will almost 
 * certainly be broken (repeatedly) as the API evolves.
 * </p>
 */
public abstract class TargetResource extends PlatformObject implements ITargetResource {

	/**
	 * Relative path that indicates the parent of this resource.
	 */
	public static final String PARENT_RELATIVE_PATH = ".."; //$NON-NLS-1$
	
	private Site site;

	/**
	 * Create the target resoource corresponding the the resource
	 * located at the govin relative path from the given site
	 */
	public TargetResource(Site site) {
		this.site = site;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.target.ITargetResource#getSite()
	 */
	public Site getSite() {
		return site;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.ITargetResource#getName()
	 */
	public String getName() {
		return Utils.getLastSegment(getRelativePath());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.ITargetResource#getFullPath()
	 */
	public String getFullPath() {
		return Utils.getFullPath(getSite(), getRelativePath());
	}
	
	/**
	 * Return the resource's location as a URL. A <code>null</code>
	 * is returned if the location (as returned by <code>getFullPath()</code>)
	 * is not a valid URL. Subclasses can override.
	 * @return the resource's location as a URL or <code>null</code>
	 */
	public URL toUrl() {
		try {
			return new URL(getFullPath());
		} catch (MalformedURLException e) {
			return null;
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.target.ITargetResource#getParent()
	 */
	public ITargetResource getParent() {
		if (getRelativePath().length() == 0)
			return null;
		return createResource(PARENT_RELATIVE_PATH, true /* isDirectory */);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.target.ITargetResource#getFile(java.lang.String)
	 */
	public ITargetResource getFile(String name) {
		return createResource(name, false /* isDirectory */);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.ITargetResource#getFolder(java.lang.String)
	 */
	public ITargetResource getFolder(String name) {
		return createResource(name, true /* isDirectory */);
	}
	
	protected abstract ITargetResource createResource(String relativePath, boolean isDirectory);
	
	/* (non-Javadoc)
	 * @see org.eclipse.target.ITargetResource#isSupported(org.eclipse.core.runtime.QualifiedName)
	 */
	public boolean isSupported(QualifiedName property) {
		QualifiedName[] supported = getSupportedProperties();
		for (int i = 0; i < supported.length; i++) {
			QualifiedName name = supported[i];
			if (name == property) {
				return true;
			}
		}
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.ITargetResource#getSupportedProperties()
	 */
	public QualifiedName[] getSupportedProperties() {
		return new QualifiedName[] { LAST_MODIFIED_TIMESTAMP_PROPERTY, SIZE_PROPERTY, CONTENT_IDENTIFIER_PROPERTY };
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj) {
		if (obj instanceof ITargetResource) {
			ITargetResource resource = (ITargetResource)obj;
			// Resource's are equal if there full paths are the same and they come from
			// the same site type
			return getSite().getTypeId().equals(resource.getSite().getTypeId())
					&& resource.getFullPath().equals(getFullPath());
		}
		return super.equals(obj);
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return getFullPath().hashCode();
	}
}
