/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.target.internal.ui.actions;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.window.Window;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.target.ITargetResource;
import org.eclipse.target.internal.*;
import org.eclipse.target.internal.ui.TargetAction;

/**
 * Action to create a new remote folder.
 */
public class CreateNewFolderAction extends TargetAction {

	/**
	 * @see TargetAction#isEnabled()
	 */
	protected boolean isEnabled() throws CoreException {
		return getSelectedRemoteFolders().length == 1;
	}

    
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
	 */
	public void run(IAction action) {
		try {
			Utils.runWithProgressDialog(getShell(), true /* cancelable */, new IRunnableWithProgress() {
				public void run(IProgressMonitor monitor)
					throws InvocationTargetException, InterruptedException {
						try {
							createDir(getShell(), getSelectedRemoteFolders()[0], new String());
						} catch (CoreException e) {
							throw new InvocationTargetException(e);
						}
				}
			});
		} catch (InvocationTargetException e) {
			handle(e, TargetMessages.Error, TargetMessages.CreateNewFolderAction_errorCreatingFolder); 
		} catch (InterruptedException e) {
		}
	}
	
	/**
	 * Throws a TeamException if one occured.
	 * Returns null if the operation was cancelled or an exception occured
	 */
	public static ITargetResource createDir(final Shell shell, final ITargetResource parent, final String defaultName) throws CoreException {
		final ITargetResource[] newFolder = new ITargetResource[] {null};
		try {				
			Utils.runWithProgressDialog(shell, true, new IRunnableWithProgress() {
				public void run(final IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
					try {
						monitor.beginTask(TargetMessages.CreateNewFolderAction_creatingFolder, IProgressMonitor.UNKNOWN); 
						final String[] folderName = new String[] {null};
						final String suggestedName = getSuggestedFolderName(parent, TargetPlugin.subMonitorFor(monitor, 0), defaultName);
						shell.getDisplay().syncExec(new Runnable() {
							public void run() {
								InputDialog dialog = new InputDialog(shell, 
									TargetMessages.CreateNewFolderAction_title,  
									TargetMessages.CreateNewFolderAction_message,  
									suggestedName,
									null);
								TargetPlugin.checkCanceled(monitor);
								if(dialog.open() == Window.OK) {
									folderName[0] = dialog.getValue();
								}
							}
						});
						if(folderName[0] != null) {
							newFolder[0] = parent.getFolder(folderName[0]);
							newFolder[0].mkdirs(TargetPlugin.subMonitorFor(monitor, 0));
						}
					} catch(CoreException e) {
						throw new InvocationTargetException(e);
					} finally {
						monitor.done();
					}
				}
			});
		} catch(InvocationTargetException e) {
			if (e.getTargetException() instanceof CoreException) {
				throw (CoreException)e.getTargetException();
			}
			Utils.handle(shell, e);
		} catch(InterruptedException e) {
		}
		return newFolder[0];
	}
	
	protected static String getSuggestedFolderName(ITargetResource parent, IProgressMonitor monitor, String defaultName) throws CoreException {
		monitor.subTask(TargetMessages.CreateNewFolderAction_suggestedNameProgress); 
		ITargetResource[] members = parent.members(monitor);
		ArrayList names = new ArrayList();
		
		String suggestedFolderName = defaultName;
		
		boolean nameUsed = false;
		
		for (int i = 0; i < members.length; i++) {
			String existingName = members[i].getName();
			names.add(existingName);  // cache them for next round since faster
			if(existingName.equals(suggestedFolderName)) {
				nameUsed = true;
			}							
		}
		
		if(!nameUsed)
			return suggestedFolderName;
			
		//Otherwise, keep proposing new names by incrementing a postfix name index until we find one not used
		int postfix = 1;

		String baseName = suggestedFolderName;

		do {			
			nameUsed = false;
			for (int i = 0; i < members.length && !nameUsed; i++) {
				suggestedFolderName = NLS.bind(TargetMessages.CreateNewFolderAction_suggestedNameConcat, new Object[] { baseName, String.valueOf(postfix) }); 
				String existingName = (String) names.get(i);
				
				if(existingName.equals(suggestedFolderName)) {
					postfix = postfix + 1;
					nameUsed = true;
				}							
			}
		} while(nameUsed);
		return suggestedFolderName;
	}
}
