/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.internal.target;

import java.io.*;
import java.util.*;
import java.util.List;

import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import org.eclipse.core.runtime.jobs.ILock;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.window.Window;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;
import org.eclipse.target.*;
import org.eclipse.team.core.TeamException;
import org.eclipse.team.internal.core.TeamPlugin;
import org.eclipse.team.internal.ui.dialogs.DetailsDialog;
import org.eclipse.team.target.DeploymentProvider;
import org.eclipse.team.target.IDeploymentProviderManager;

public class DeploymentProviderManager implements IDeploymentProviderManager, IResourceChangeListener, ITargetListener  {
	
	// key for remembering if state has been loaded for a project
	private final static QualifiedName STATE_LOADED_KEY = new QualifiedName("org.eclipse.target.team", "state_restored_key"); //$NON-NLS-1$ //$NON-NLS-2$
	
	// {project -> list of Mapping}
	private Map mappings = new HashMap(5);

	// registry for deployment provider extensions
	private DeploymentProviderRegistry registry;
	
	// lock to ensure that map/unmap and getProvider support concurrency
	private static final ILock mappingLock = Platform.getJobManager().newLock();
	
	//	persistence constants
	private final static String CTX_PROVIDERS = "deploymentProviders"; //$NON-NLS-1$
	private final static String CTX_PROVIDER = "provider"; //$NON-NLS-1$
	private final static String CTX_ID = "id"; //$NON-NLS-1$
	private final static String CTX_PATH = "container_path"; //$NON-NLS-1$
	private final static String CTX_PROVIDER_DATA = "data"; //$NON-NLS-1$
	private final static String FILENAME = ".deployments"; //$NON-NLS-1$
	
	private class AlreadyMappedDialog extends DetailsDialog {
		private org.eclipse.swt.widgets.List detailsList;
		private Button unmap;
		private final DeploymentProvider[] providers;
		
		public AlreadyMappedDialog(Shell shell, DeploymentProvider[] providers) {
			super(shell, TeamTargetMessages.SiteExplorerView_unmapDialogTitle); 
			this.providers = providers;
			setImageKey(DLG_IMG_WARNING);
		}
			
		protected Composite createDropDownDialogArea(Composite parent) {
			// create a composite with standard margins and spacing
			Composite composite = new Composite(parent, SWT.NONE);
			GridLayout layout = new GridLayout();
			layout.marginHeight = convertVerticalDLUsToPixels(IDialogConstants.VERTICAL_MARGIN);
			layout.marginWidth = convertHorizontalDLUsToPixels(IDialogConstants.HORIZONTAL_MARGIN);
			layout.verticalSpacing = convertVerticalDLUsToPixels(IDialogConstants.VERTICAL_SPACING);
			layout.horizontalSpacing = convertHorizontalDLUsToPixels(IDialogConstants.HORIZONTAL_SPACING);
			composite.setLayout(layout);
			composite.setLayoutData(new GridData(GridData.FILL_BOTH));
			composite.setFont(parent.getFont());
			
			detailsList = new org.eclipse.swt.widgets.List(composite, SWT.BORDER | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);	 
			GridData data = new GridData ();		
			data.heightHint = 75;
			data.horizontalAlignment = GridData.FILL;
			data.grabExcessHorizontalSpace = true;
			detailsList.setLayoutData(data);
			
			for (int i = 0; i < providers.length; i++) {
				DeploymentProvider provider = providers[i];
				detailsList.add(NLS.bind(TeamTargetMessages.SiteExplorerView_mappedRoots, new String[] { provider.getMappedContainer().getFullPath().toString(), provider.getDisplayName() })); 
			}			
			return composite;
		}

		protected void createMainDialogArea(Composite composite) {
			Label label = new Label(composite, SWT.WRAP);
			label.setText(TeamTargetMessages.SiteExplorerView_projectsAlreadyMapped); 
			GridData data = new GridData(
				GridData.GRAB_HORIZONTAL |
				GridData.GRAB_VERTICAL |
				GridData.HORIZONTAL_ALIGN_FILL |
				GridData.VERTICAL_ALIGN_CENTER);
			data.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
			label.setLayoutData(data);
			label.setFont(composite.getFont());
						
			unmap = new Button(composite, SWT.CHECK);
			unmap.setText(TeamTargetMessages.SiteExplorerView_unmapProjectsAndDisconnect); 
			unmap.setSelection(false);
			unmap.addListener(SWT.Selection, new Listener() {				
				public void handleEvent(Event event) {
					updateEnablements();
				}
			});
			setPageComplete(false);
			updateEnablements();
		}

		protected void updateEnablements() {
			setPageComplete(unmap.getSelection());
		}
	}
	
	static class Mapping {
		private DeploymentProviderDescriptor descriptor;
		private DeploymentProvider provider;
		private IContainer container;
		private IMemento savedState;
		
		Mapping(DeploymentProviderDescriptor descriptor, IContainer container) {
			this.descriptor = descriptor;
			this.container = container;
		}
		public DeploymentProvider getProvider() throws TeamException {
			if(provider == null) {
				try {
					this.provider = descriptor.createProvider();
					this.provider.setContainer(container);
					this.provider.restoreState(savedState);
					this.savedState = null;
				} catch (CoreException e) {
					throw TeamException.asTeamException(e);
				}				
			}
			return provider;
		}
		public void setProvider(DeploymentProvider provider) {
			this.provider = provider;
			this.savedState = null;
		}
		public IContainer getContainer() {
			return container;
		}
		public DeploymentProviderDescriptor getDescription() {
			return descriptor;
		}
		public void setProviderState(IMemento savedState) {
			this.savedState = savedState;
		}
	}
	
	public DeploymentProviderManager() {
		registry = new DeploymentProviderRegistry();
		Target.getSiteManager().addSiteListener(this);
	}
	
	public void map(IContainer container, DeploymentProvider deploymentProvider) throws TeamException {
		try {
			mappingLock.acquire();
			if (!deploymentProvider.isMultipleMappingsSupported()) {
				// don't allow is overlapping deployment providers of the same type
				checkOverlapping(container, deploymentProvider.getID());
			}
			
			// extension point descriptor must exist
			DeploymentProviderDescriptor descriptor = registry.find(deploymentProvider.getID());		
			if(descriptor == null) {
				throw new TeamException(NLS.bind(TeamTargetMessages.DeploymentProviderManager_10, new String[] { deploymentProvider.getID() })); 
			}
			
			// create the new mapping
			Mapping m = internalMap(container, descriptor);
			m.setProvider(deploymentProvider);
			deploymentProvider.setContainer(container);
			deploymentProvider.init();
			
			saveState(container.getProject());
			// TODO: what kind of event is generated when one is mapped?	
		} finally {
			mappingLock.release();
		}
	}
	
	public void unmap(IContainer container, DeploymentProvider teamProvider) throws TeamException {
		try {
			mappingLock.acquire();
			IProject project = container.getProject();
			List projectMaps = internalGetMappings(container);
			Mapping[] m = internalGetMappingsFor(container, teamProvider.getID());
			for (int i = 0; i < m.length; i++) {
				Mapping mapping = m[i];
				if (mapping.getProvider() == teamProvider) {
					projectMaps.remove(mapping);
					if(projectMaps.isEmpty()) {
						mappings.remove(project);
					}
				}
			}
			
			// dispose of provider
			teamProvider.dispose();
			saveState(container.getProject());
			
			// TODO: what kind of event is sent when unmapped?
		} finally {
			mappingLock.release();
		}
	}
	
	public DeploymentProvider[] getMappings(IResource resource) {
		List projectMappings = internalGetMappings(resource);
		String fullPath = resource.getFullPath().toString();
		List result = new ArrayList();
		if(projectMappings != null) {
			for (Iterator it = projectMappings.iterator(); it.hasNext();) {
				Mapping m = (Mapping) it.next();
				if(fullPath.startsWith(m.getContainer().getFullPath().toString())) {
					try {
						// lazy initialize of provider must be supported
						// TODO: It is possible that the provider has been unmap concurrently
						result.add(m.getProvider());
					} catch (CoreException e) {
						TeamPlugin.log(e);
					}
				}
			}
		}
		return (DeploymentProvider[]) result.toArray(new DeploymentProvider[result.size()]);
	}
	
	public DeploymentProvider[] getMappings(IResource resource, String id) {
		Mapping[] m = internalGetMappingsFor(resource, id);
		List result = new ArrayList();
		for (int i = 0; i < m.length; i++) {
			Mapping mapping = m[i];
			try {
				// lazy initialize of provider must be supported
				// TODO: It is possible that the provider has been unmap concurrently
				result.add(mapping.getProvider());
			} catch (TeamException e) {
				TeamPlugin.log(e);
			}
		}
		
		DeploymentProvider[] providers = (DeploymentProvider[]) result.toArray(new DeploymentProvider[result.size()]);
		// Ensure that multiple providers are not mapped if it is not supported 
		// by the provider type. This could occur if the deployment configuration 
		// was loaded from a repository or modified manually
		if (providers.length > 1 && !providers[0].isMultipleMappingsSupported()) {
			// Log and ignore all but one of the mappings
			TeamPlugin.log(IStatus.WARNING, NLS.bind(TeamTargetMessages.DeploymentProviderManager_12, new String[] { resource.getFullPath().toString(), id }), null); 
			return new DeploymentProvider[] { providers[0] };
		}
		return providers;
	}
	
	public boolean getMappedTo(IResource resource, String id) {
		return internalGetMappingsFor(resource, id).length > 0;
	}
	
	private void checkOverlapping(IContainer container, String id) throws TeamException {
		List projectMappings = internalGetMappings(container);
		String fullPath = container.getFullPath().toString();
		if(projectMappings != null) {
			for (Iterator it = projectMappings.iterator(); it.hasNext();) {
				Mapping m = (Mapping) it.next();
				String first = m.getContainer().getFullPath().toString();
				if(fullPath.startsWith(first) || first.startsWith(fullPath)) {
					if (m.getDescription().getId().equals(id)) {
						throw new TeamException(NLS.bind(TeamTargetMessages.DeploymentProviderManager_13, new String[] { container.getFullPath().toString(), m.getDescription().getId() })); 
					}
				}
			}
		}
	}
	
	private Mapping internalMap(IContainer container, DeploymentProviderDescriptor description) {
		Mapping newMapping = new Mapping(description, container);
		return internalMap(container, newMapping);
	}
	
	private Mapping internalMap(IContainer container, Mapping newMapping) {
		IProject project = container.getProject();
		List projectMaps = (List)mappings.get(project);
		if(projectMaps == null) {
			projectMaps = new ArrayList();
			mappings.put(project, projectMaps);
		}
		projectMaps.add(newMapping);
		return newMapping;
	}

	/*
	 * Loads all the mappings associated with the resource's project.
	 */
	private List internalGetMappings(IResource resource) {
		try {
			mappingLock.acquire();
			IProject project = resource.getProject();
			List m = (List)mappings.get(project);
			try {
				if(project.getSessionProperty(STATE_LOADED_KEY) != null) {
					return m;
				}
				Mapping[] projectMappings = loadMappings(project);
				for (int i = 0; i < projectMappings.length; i++) {
					Mapping mapping = projectMappings[i];
					internalMap(mapping.getContainer(), mapping);
				}
				
				project.setSessionProperty(STATE_LOADED_KEY, new Object());
			} catch (TeamException e) {
			} catch (CoreException e) {
			}		
			return (List)mappings.get(project);
		} finally {
			mappingLock.release();
		}
	}
	
	private Mapping[] internalGetMappingsFor(IResource resource, String id) {
		List projectMappings = internalGetMappings(resource);
		List result = new ArrayList();
		String fullPath = resource.getFullPath().toString();
		if(projectMappings != null) {
			for (Iterator it = projectMappings.iterator(); it.hasNext();) {
				Mapping m = (Mapping) it.next();				
				// mapping can be initialize without having provider loaded yet!
				if(m.getDescription().getId().equals(id) && fullPath.startsWith(m.getContainer().getFullPath().toString())) {
					result.add(m);
				}
			}
		}
		return (Mapping[]) result.toArray(new Mapping[result.size()]);
	}
	
	/**
	 * Saves a file containing the list of participant ids that are registered with this
	 * manager. Each participant is also given the chance to save it's state. 
	 */
	private void saveState(IProject project) throws TeamException {
		File file = getNonsharedSettingsFile(project);
		try {
			XMLMemento xmlMemento = XMLMemento.createWriteRoot(CTX_PROVIDERS);	
			List providers = (List)mappings.get(project);
			if(providers == null) {
				if (file.exists()) {
					file.delete();
				}
			} else {
				for (Iterator it2 = providers.iterator(); it2.hasNext(); ) {
					Mapping mapping = (Mapping) it2.next();
					IMemento node = xmlMemento.createChild(CTX_PROVIDER);
					node.putString(CTX_ID, mapping.getDescription().getId());
					node.putString(CTX_PATH, mapping.getContainer().getProjectRelativePath().toString());
					mapping.getProvider().saveState(node.createChild(CTX_PROVIDER_DATA));
				}				
				Writer writer = new BufferedWriter(new FileWriter(file));
				try {
					xmlMemento.save(writer);
				} finally {
					writer.close();
				}
			}
		} catch (IOException e) {
			throw new TeamException(NLS.bind(TeamTargetMessages.DeploymentProviderManager_15, new String[] { project.getName() }), e); 
		} catch(CoreException ce) {
			throw TeamException.asTeamException(ce);
		}
	}
	
	/**
	 * @param project
	 * @return
	 */
	private File getNonsharedSettingsFile(IProject project) {
		IPath metaPath = project.getWorkingLocation(TeamPlugin.getPlugin().getBundle().getSymbolicName());
		metaPath = metaPath.append(FILENAME);
		File file = metaPath.toFile();
		return file;
	}

	/*
	 * Load the mappings for the given project and return them.
	 */
	private Mapping[] loadMappings(IProject project) throws TeamException, CoreException {
		File file = getNonsharedSettingsFile(project);
		if(! file.exists()) {
			// The file may have been deleted before our delta listener was loaded.
			// If there are any deployments stored in the meta data area, dispose of them
			// TODO: See if there were any before and dispose of them
			return new Mapping[0];
		}
		Reader reader;
		try {
			reader = new BufferedReader(new FileReader(file));
		} catch (FileNotFoundException e) {
			return new Mapping[0];
		}
		return loadMappings(project, reader);
	}

	private Mapping[] loadMappings(IProject project, Reader reader) throws TeamException {
		try {
			IMemento memento = XMLMemento.createReadRoot(reader);
			IMemento[] providers = memento.getChildren(CTX_PROVIDER);
			List projectMappings = new ArrayList();
			for (int i = 0; i < providers.length; i++) {
				IMemento memento2 = providers[i];			
				String id = memento2.getString(CTX_ID);
				IPath location = new Path(memento2.getString(CTX_PATH));
				
				if(! project.exists(location)) {
					TeamPlugin.log(IStatus.ERROR, NLS.bind(TeamTargetMessages.DeploymentProviderManager_16, new String[] { location.toString(), project.getName() }), null); 
				}
				IResource resource = location.isEmpty() ? (IContainer)project : project.findMember(location);
				if (resource.getType() == IResource.FILE) {
					TeamPlugin.log(IStatus.ERROR, NLS.bind(TeamTargetMessages.DeploymentProviderManager_17, new String[] { location.toString(), project.getName() }), null); 
				}
				IContainer container = (IContainer)resource;
				DeploymentProviderDescriptor desc = registry.find(id);				
				if(desc != null) {
					Mapping m = new Mapping(desc, container);
					m.setProviderState(memento2.getChild(CTX_PROVIDER_DATA));
					projectMappings.add(m);
				} else {
					TeamPlugin.log(IStatus.ERROR, NLS.bind("Could not retore deployment provider of type {0}. There is no provider of that type registered.", new Object[] { id }), null); //$NON-NLS-1$ 
				}
			}
			return (Mapping[]) projectMappings.toArray(new Mapping[projectMappings.size()]);
		} finally {
			try {
				reader.close();
			} catch (IOException e) {
				// ignore
			}
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.core.IDeploymentProviderManager#getDeploymentProviderRoots(java.lang.String)
	 */
	public IResource[] getDeploymentProviderRoots(String id) {
		IProject[] projects = ResourcesPlugin.getWorkspace().getRoot().getProjects();
		Set roots = new HashSet();
		for (int i = 0; i < projects.length; i++) {
			IProject project = projects[i];
			List mappings = internalGetMappings(project);
			if (mappings != null) {
				for (Iterator iter = mappings.iterator(); iter.hasNext();) {
					Mapping mapping = (Mapping) iter.next();
					if (id == null || mapping.getDescription().getId().equals(id)) {
						roots.add(mapping.getContainer());
					}
				}
			}
		}
		return (IResource[]) roots.toArray(new IResource[roots.size()]);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.resources.IResourceChangeListener#resourceChanged(org.eclipse.core.resources.IResourceChangeEvent)
	 */
	public void resourceChanged(IResourceChangeEvent event) {
		processDelta(event.getDelta());
	}

	private void processDelta(IResourceDelta delta) {
		IResource resource = delta.getResource();
		int kind = delta.getKind();

		if (resource.getType() == IResource.PROJECT) {
			// Handle a deleted project
			if (((kind & IResourceDelta.REMOVED) != 0)) {
				handleProjectRemoval((IProject)resource);
				return;
			}
			// Handle a closed project
			if ((delta.getFlags() & IResourceDelta.OPEN) != 0 && !((IProject) resource).isOpen()) {
				handleProjectClose((IProject)resource);
				return;
			}
		}

		if (((kind & IResourceDelta.REMOVED) != 0) && (resource.getType() == IResource.FOLDER)) {
			handleFolderRemoval((IFolder)resource);
		}

		// Handle changed children
		IResourceDelta[] affectedChildren = delta.getAffectedChildren(IResourceDelta.CHANGED | IResourceDelta.REMOVED | IResourceDelta.ADDED);
		for (int i = 0; i < affectedChildren.length; i++) {
			processDelta(affectedChildren[i]);
		}
	}

	private void handleFolderRemoval(IFolder folder) {
		DeploymentProvider[] providers = getMappings(folder);
		for (int i = 0; i < providers.length; i++) {
			DeploymentProvider provider = providers[i];
			if (provider.getMappedContainer().equals(folder)) {
				try {
					unmap(folder, provider);
				} catch (TeamException e) {
					TeamPlugin.log(e);
				}
			}
		}
	}

	private void handleProjectClose(IProject project) {
		mappings.remove(project);
	}

	private void handleProjectRemoval(IProject project) {
		mappings.remove(project);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.ITargetListener#siteAdded(org.eclipse.target.Site)
	 */
	public void siteAdded(Site site) {
		// Don't care
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.ITargetListener#siteRemoved(org.eclipse.target.Site)
	 */
	public void siteRemoved(Site site) {
		DeploymentProvider[] providers = getProviders(new Site[] { site });
		for (int i = 0; i < providers.length; i++) {
			DeploymentProvider provider = providers[i];
			try {
				unmap(provider.getMappedContainer(), provider);
			} catch (TeamException e) {
				TeamTargetPlugin.log(e);
			}
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.ITargetListener#aboutToRemove(org.eclipse.target.Site, org.eclipse.swt.widgets.Shell, org.eclipse.core.runtime.IProgressMonitor)
	 */
	public boolean aboutToRemove(Site[] sites, Shell shell) {
		DeploymentProvider[] providers = getProviders(sites);
		if( providers.length > 0 ) {
			AlreadyMappedDialog dialog = new AlreadyMappedDialog(shell, providers);
			return (dialog.open() == Window.OK);
		}
		return true;
	}

	private DeploymentProvider[] getProviders(Site[] sites) {
		List mappedToSite = new ArrayList();
		IResource[] roots = getDeploymentProviderRoots(null);
		for (int j = 0; j < sites.length; j++) {
			Site site = sites[j];
			for (int k = 0; k < roots.length; k++) {
				IResource resource = roots[k];
				DeploymentProvider[] providers = getMappings(resource);
				for (int i = 0; i < providers.length; i++) {
					DeploymentProvider provider = providers[i];
					if (provider.getSite().equals(site)) {
						mappedToSite.add(provider);
					}
				}
			}
		}
		return (DeploymentProvider[]) mappedToSite.toArray(new DeploymentProvider[mappedToSite.size()]);
	}

}
